@@ -1,14 +1,13 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * preview.c
- * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * preview.c is free software.
  *
  * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * GNU General Public License version 2, as published by the Free Software
+ * Foundation.
  *
  * preview.c is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,12 +21,14 @@
  *  Boston, MA  02110-1301, USA.
  */
 
+#include "ghbcompat.h"
+#include "config.h"
+
 #include <unistd.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
-#include "ghbcompat.h"
 
 #if defined(_ENABLE_GST)
 #include <gst/gst.h>
@@ -36,13 +37,14 @@
 #endif
 
 #include "settings.h"
+#include "jobdict.h"
 #include "presets.h"
 #include "callbacks.h"
 #include "hb-backend.h"
 #include "preview.h"
 #include "values.h"
-#include "queuehandler.h"
-#include "hb.h"
+#include "title-add.h"
+#include "handbrake/handbrake.h"
 
 #define PREVIEW_STATE_IMAGE 0
 #define PREVIEW_STATE_LIVE 1
@@ -74,14 +76,15 @@ struct preview_s
     gint        live_id;
     gchar     * current;
     gint        live_enabled;
+    gboolean    is_fullscreen;
 };
 
 #if defined(_ENABLE_GST)
 G_MODULE_EXPORT gboolean live_preview_cb(GstBus *bus, GstMessage *msg, gpointer data);
 #endif
 
-void
-ghb_screen_par(signal_user_data_t *ud, gint *par_n, gint *par_d)
+static void
+screen_par (signal_user_data_t *ud, gint *par_n, gint *par_d)
 {
     // Assume 1:1
     // I could get it from GtkWindow->GdkScreen monitor methods.
@@ -96,7 +99,7 @@ ghb_par_scale(signal_user_data_t *ud, gint *width, gint *height, gint par_n, gin
     gint disp_par_n, disp_par_d;
     gint64 num, den;
 
-    ghb_screen_par(ud, &disp_par_n, &disp_par_d);
+    screen_par(ud, &disp_par_n, &disp_par_d);
     if (disp_par_n < 1 || disp_par_d < 1)
     {
         disp_par_n = 1;
@@ -111,35 +114,83 @@ ghb_par_scale(signal_user_data_t *ud, gint *width, gint *height, gint par_n, gin
         *height = *height * den / num;
 }
 
-void
-preview_set_size(signal_user_data_t *ud, int width, int height)
-{
-    ud->preview->width = width;
-    ud->preview->height = height;
-}
-
-void
+static void
 preview_set_render_size(signal_user_data_t *ud, int width, int height)
 {
-    GtkWidget     * widget;
+    GtkWidget     * widget, *frame, *reset;
     GtkWindow     * window;
-    GdkGeometry     geo;
+    gint            s_w, s_h;
+    gint            factor;
+    gfloat          ratio = 1.0;
 
-    widget = GHB_WIDGET (ud->builder, "preview_image");
-    gtk_widget_set_size_request(widget, width, height);
     window = GTK_WINDOW(GHB_WIDGET(ud->builder, "preview_window"));
-    gtk_window_unmaximize(window);
-    gtk_window_resize(window, width, height);
-    geo.min_aspect = (double)(width - 4) / height;
-    geo.max_aspect = (double)(width + 4) / height;
-    geo.width_inc = geo.height_inc = 2;
-    gtk_window_set_geometry_hints(window, NULL, &geo,
-                                  GDK_HINT_ASPECT|GDK_HINT_RESIZE_INC);
+    widget = GHB_WIDGET (ud->builder, "preview_image");
+    frame = GHB_WIDGET (ud->builder, "preview_image_frame");
+
+    if (ghb_dict_get_bool(ud->prefs, "reduce_hd_preview"))
+        factor = 90;
+    else
+        factor = 100;
+
+    ghb_monitor_get_size(GHB_WIDGET(ud->builder, "hb_window"), &s_w, &s_h);
+
+    if (s_w > 0 && s_h > 0)
+    {
+        int orig_w = width;
+        int orig_h = height;
+
+        if (width > s_w * factor / 100)
+        {
+            width = s_w * factor / 100;
+            height = height * width / orig_w;
+        }
+        if (height > s_h * factor / 100)
+        {
+            height = s_h * factor / 100;
+            width = orig_w * height / orig_h;
+        }
+    }
+    if (height && width)
+        ratio = (gfloat) width / height;
+
+    gtk_widget_set_size_request(widget, width, height);
+    gtk_aspect_frame_set(GTK_ASPECT_FRAME(frame), 0.5, 0.5, ratio, FALSE);
+
+    if (ud->preview->is_fullscreen)
+    {
+        reset = GHB_WIDGET(ud->builder, "preview_reset");
+        gtk_widget_hide(reset);
+    }
+    else
+    {
+        gtk_window_unmaximize(window);
+        gtk_window_resize(window, width, height);
+    }
+    gtk_widget_set_size_request(widget, -1, -1);
 
     ud->preview->render_width = width;
     ud->preview->render_height = height;
 }
 
+static void
+preview_set_size(signal_user_data_t *ud, int width, int height)
+{
+    if (height == ud->preview->width &&
+        width == ud->preview->height)
+    {
+        // Rotation happened, fix up render size
+        preview_set_render_size(ud, ud->preview->render_height,
+                                ud->preview->render_width);
+    }
+    else if (width != ud->preview->width ||
+             height != ud->preview->height)
+    {
+        preview_set_render_size(ud, width, height);
+    }
+    ud->preview->width = width;
+    ud->preview->height = height;
+}
+
 void
 ghb_preview_init(signal_user_data_t *ud)
 {
@@ -161,8 +212,8 @@ ghb_preview_init(signal_user_data_t *ud)
     ud->preview->vsink = gst_element_factory_make("gdkpixbufsink", "pixsink");
     if (ud->preview->play == NULL || ud->preview->vsink == NULL)
     {
-        g_message("Couldn't initialize gstreamer. Disabling live preview.");
-        GtkWidget *widget = GHB_WIDGET(ud->builder, "live_preview_box");
+        g_warning("Couldn't initialize gstreamer. Disabling live preview.");
+        widget = GHB_WIDGET(ud->builder, "live_preview_box");
         gtk_widget_hide (widget);
         widget = GHB_WIDGET(ud->builder, "live_preview_duration_box");
         gtk_widget_hide (widget);
@@ -200,7 +251,7 @@ ghb_preview_cleanup(signal_user_data_t *ud)
 }
 
 #if defined(_ENABLE_GST)
-void
+static void
 live_preview_start(signal_user_data_t *ud)
 {
     GtkImage *img;
@@ -212,7 +263,7 @@ live_preview_start(signal_user_data_t *ud)
     img = GTK_IMAGE(GHB_WIDGET(ud->builder, "live_preview_play_image"));
     if (!ud->preview->encoded[ud->preview->frame])
     {
-        gtk_image_set_from_icon_name(img, GHB_PLAY_ICON, GTK_ICON_SIZE_BUTTON);
+        ghb_image_set_from_icon_name(img, GHB_PLAY_ICON, GHB_ICON_SIZE_BUTTON);
         gst_element_set_state(ud->preview->play, GST_STATE_NULL);
         ud->preview->pause = TRUE;
         return;
@@ -225,7 +276,7 @@ live_preview_start(signal_user_data_t *ud)
 #else
         uri = g_strdup_printf("file://%s", ud->preview->current);
 #endif
-        gtk_image_set_from_icon_name(img, GHB_PAUSE_ICON, GTK_ICON_SIZE_BUTTON);
+        ghb_image_set_from_icon_name(img, GHB_PAUSE_ICON, GHB_ICON_SIZE_BUTTON);
         ud->preview->state = PREVIEW_STATE_LIVE;
         g_object_set(G_OBJECT(ud->preview->play), "uri", uri, NULL);
         g_free(uri);
@@ -234,7 +285,7 @@ live_preview_start(signal_user_data_t *ud)
     ud->preview->pause = FALSE;
 }
 
-void
+static void
 live_preview_pause(signal_user_data_t *ud)
 {
     GtkImage *img;
@@ -243,13 +294,13 @@ live_preview_pause(signal_user_data_t *ud)
         return;
 
     img = GTK_IMAGE(GHB_WIDGET(ud->builder, "live_preview_play_image"));
-    gtk_image_set_from_icon_name(img, GHB_PLAY_ICON, GTK_ICON_SIZE_BUTTON);
+    ghb_image_set_from_icon_name(img, GHB_PLAY_ICON, GHB_ICON_SIZE_BUTTON);
     gst_element_set_state(ud->preview->play, GST_STATE_PAUSED);
     ud->preview->pause = TRUE;
 }
 #endif
 
-void
+static void
 live_preview_stop(signal_user_data_t *ud)
 {
     GtkImage *img;
@@ -259,7 +310,7 @@ live_preview_stop(signal_user_data_t *ud)
         return;
 
     img = GTK_IMAGE(GHB_WIDGET(ud->builder, "live_preview_play_image"));
-    gtk_image_set_from_icon_name(img, GHB_PLAY_ICON, GTK_ICON_SIZE_BUTTON);
+    ghb_image_set_from_icon_name(img, GHB_PLAY_ICON, GHB_ICON_SIZE_BUTTON);
 #if defined(_ENABLE_GST)
     gst_element_set_state(ud->preview->play, GST_STATE_NULL);
 #endif
@@ -315,7 +366,7 @@ caps_set(GstCaps *caps, signal_user_data_t *ud)
         par_n = gst_value_get_fraction_numerator(par);
         par_d = gst_value_get_fraction_denominator(par);
 
-        ghb_screen_par(ud, &disp_par_n, &disp_par_d);
+        screen_par(ud, &disp_par_n, &disp_par_d);
         gst_video_calculate_display_ratio(
             &num, &den, width, height, par_n, par_d, disp_par_n, disp_par_d);
 
@@ -327,13 +378,10 @@ caps_set(GstCaps *caps, signal_user_data_t *ud)
         preview_set_size(ud, width, height);
         if (ghb_dict_get_bool(ud->prefs, "reduce_hd_preview"))
         {
-            GdkWindow *window;
             gint s_w, s_h;
 
-            window = gtk_widget_get_window(
-                        GHB_WIDGET(ud->builder, "preview_window"));
-            ghb_monitor_get_size(window, &s_w, &s_h);
-
+            ghb_monitor_get_size(GHB_WIDGET(ud->builder, "preview_window"),
+                                 &s_w, &s_h);
             if (s_w > 0 && s_h > 0)
             {
                 if (width > s_w * 80 / 100)
@@ -486,7 +534,7 @@ live_preview_cb(GstBus *bus, GstMessage *msg, gpointer data)
                             "Audio or Video may not play as expected\n\n%s"),
                             desc);
                 ghb_message_dialog(hb_window, GTK_MESSAGE_WARNING,
-                                   message, "Ok", NULL);
+                                   message, _("OK"), NULL);
                 g_free(message);
                 gst_element_set_state(ud->preview->play, GST_STATE_PLAYING);
             }
@@ -645,6 +693,7 @@ live_preview_start_cb(GtkWidget *xwidget, signal_user_data_t *ud)
 
     tmp_dir = ghb_get_tmp_dir();
     name = g_strdup_printf("%s/live%02d", tmp_dir, ud->preview->frame);
+    free(tmp_dir);
     if (ud->preview->current)
         g_free(ud->preview->current);
     ud->preview->current = name;
@@ -803,6 +852,26 @@ live_preview_seek_cb(GtkWidget *widget, signal_user_data_t *ud)
 #endif
 }
 
+G_MODULE_EXPORT void
+preview_fullscreen_action_cb(GSimpleAction *action, GVariant *param,
+                             signal_user_data_t *ud)
+{
+    gboolean state = g_variant_get_boolean(param);
+    GtkWindow *window = GTK_WINDOW(GHB_WIDGET(ud->builder, "preview_window"));
+
+    if (state != ud->preview->is_fullscreen)
+        g_simple_action_set_state(action, param);
+
+    if (!ud->preview->is_fullscreen)
+    {
+        gtk_window_fullscreen(window);
+    }
+    else
+    {
+        gtk_window_unfullscreen(window);
+    }
+}
+
 static void _draw_pixbuf(signal_user_data_t * ud, cairo_t *cr, GdkPixbuf *pix)
 {
     int pix_width, pix_height, hoff, voff;
@@ -810,7 +879,7 @@ static void _draw_pixbuf(signal_user_data_t * ud, cairo_t *cr, GdkPixbuf *pix)
     cairo_save(cr);
     cairo_rectangle(cr, 0, 0, ud->preview->render_width,
                               ud->preview->render_height);
-    cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     cairo_fill(cr);
     cairo_restore(cr);
 
@@ -857,13 +926,18 @@ static void set_mini_preview_image(signal_user_data_t *ud, GdkPixbuf * pix)
             GtkWidget * widget;
 
             widget = GHB_WIDGET (ud->builder, "preview_button_image");
+#if GTK_CHECK_VERSION(4, 4, 0)
+            gtk_picture_set_pixbuf(GTK_PICTURE(widget), scaled_preview);
+#else
             gtk_image_set_from_pixbuf(GTK_IMAGE(widget), scaled_preview);
+#endif
             g_object_unref(scaled_preview);
         }
     }
 }
 
-GdkPixbuf * do_preview_scaling(signal_user_data_t *ud, GdkPixbuf *pix)
+static GdkPixbuf *
+do_preview_scaling (signal_user_data_t *ud, GdkPixbuf *pix)
 {
     int         preview_width, preview_height, width, height;
     GdkPixbuf * scaled_preview;
@@ -922,25 +996,13 @@ GdkPixbuf * do_preview_scaling(signal_user_data_t *ud, GdkPixbuf *pix)
     return pix;
 }
 
-void
+static void
 init_preview_image(signal_user_data_t *ud)
 {
     GtkWidget *widget;
-    gint width, height;
-
-    g_debug("set_preview_button_image ()");
-    gint title_id, titleindex;
-    const hb_title_t *title;
 
     live_preview_stop(ud);
 
-    title_id = ghb_dict_get_int(ud->settings, "title");
-    title = ghb_lookup_title(title_id, &titleindex);
-    if (title == NULL && ud->preview->pix != NULL)
-    {
-        g_object_unref(ud->preview->pix);
-        ud->preview->pix = NULL;
-    }
     widget = GHB_WIDGET (ud->builder, "preview_frame");
     ud->preview->frame = ghb_widget_int(widget) - 1;
     if (ud->preview->encoded[ud->preview->frame])
@@ -965,8 +1027,7 @@ init_preview_image(signal_user_data_t *ud)
     if (ud->preview->scaled_pix != NULL)
         g_object_unref(ud->preview->scaled_pix);
 
-    ud->preview->pix = ghb_get_preview_image(title, ud->preview->frame,
-                                             ud, &width, &height);
+    ud->preview->pix = ghb_get_preview_image(ud->preview->frame, ud);
     if (ud->preview->pix == NULL)
         return;
 
@@ -1036,6 +1097,23 @@ ghb_reset_preview_image(signal_user_data_t *ud)
     gtk_widget_queue_draw(widget);
 }
 
+#if GTK_CHECK_VERSION(4, 4, 0)
+G_MODULE_EXPORT void
+preview_draw_cb(
+    GtkDrawingArea * da,
+    cairo_t        * cr,
+    int              width,
+    int              height,
+    signal_user_data_t *ud)
+{
+    if (ud->preview->scaled_pix != NULL)
+    {
+        _draw_pixbuf(ud, cr, ud->preview->scaled_pix);
+    }
+}
+
+#else
+
 G_MODULE_EXPORT gboolean
 preview_draw_cb(
     GtkWidget *widget,
@@ -1048,27 +1126,39 @@ preview_draw_cb(
     }
     return FALSE;
 }
+#endif
 
 G_MODULE_EXPORT void
-preview_button_size_allocate_cb(GtkWidget *widget, GtkAllocation *allocation, signal_user_data_t *ud)
+preview_button_size_allocate_cb(
+    GtkWidget *widget,
+#if GTK_CHECK_VERSION(4, 4, 0)
+    int width,
+    int height,
+    int baseline,
+#else
+    GdkRectangle *rect,
+#endif
+    signal_user_data_t *ud)
 {
-    g_debug("allocate %d x %d", allocation->width, allocation->height);
-    if (ud->preview->button_width == allocation->width &&
-        ud->preview->button_height == allocation->height)
+#if !GTK_CHECK_VERSION(4, 4, 0)
+    int width  = rect->width;
+    int height = rect->height;
+#endif
+    if (ud->preview->button_width  == width &&
+        ud->preview->button_height == height)
     {
         // Nothing to do. Bug out.
         g_debug("nothing to do");
         return;
     }
-    g_debug("prev allocate %d x %d", ud->preview->button_width,
-            ud->preview->button_height);
-    ud->preview->button_width = allocation->width;
-    ud->preview->button_height = allocation->height;
+    ud->preview->button_width  = width;
+    ud->preview->button_height = height;
     set_mini_preview_image(ud, ud->preview->pix);
 }
 
-void
-ghb_preview_set_visible(signal_user_data_t *ud, gboolean visible)
+G_MODULE_EXPORT void
+show_preview_action_cb(GSimpleAction *action, GVariant *value,
+                       signal_user_data_t *ud)
 {
     GtkWidget *widget;
 #if 0
@@ -1080,50 +1170,23 @@ ghb_preview_set_visible(signal_user_data_t *ud, gboolean visible)
     visible &= title != NULL;
 #endif
     widget = GHB_WIDGET(ud->builder, "preview_window");
-    if (visible)
-    {
-        gint x, y;
-        x = ghb_dict_get_int(ud->prefs, "preview_x");
-        y = ghb_dict_get_int(ud->prefs, "preview_y");
-        if (x >= 0 && y >= 0)
-            gtk_window_move(GTK_WINDOW(widget), x, y);
-        gtk_window_deiconify(GTK_WINDOW(widget));
-    }
-    gtk_widget_set_visible(widget, visible);
-}
 
-static void
-update_preview_labels(signal_user_data_t *ud, gboolean active)
-{
-    GtkToolButton *button;
+#if !GTK_CHECK_VERSION(4, 4, 0)
+    // Note: Only works on GTK3 and with X11
+    gint x, y;
+    x = ghb_dict_get_int(ud->prefs, "preview_x");
+    y = ghb_dict_get_int(ud->prefs, "preview_y");
 
-    button   = GTK_TOOL_BUTTON(GHB_WIDGET(ud->builder, "show_preview"));
-
-    if (!active)
-    {
-        gtk_tool_button_set_label(button, "Show\nPreview");
-    }
-    else
-    {
-        gtk_tool_button_set_label(button, "Hide\nPreview");
-    }
-}
-
-G_MODULE_EXPORT void
-show_preview_action_cb(GSimpleAction *action, GVariant *value,
-                       signal_user_data_t *ud)
-{
-    gboolean state = g_variant_get_boolean(value);
-
-    g_simple_action_set_state(action, value);
-    ghb_preview_set_visible(ud, state);
-    update_preview_labels(ud, state);
+    if (x >= 0 && y >= 0)
+        gtk_window_move(GTK_WINDOW(widget), x, y);
+#endif
+    gtk_window_present(GTK_WINDOW(widget));
 }
 
 G_MODULE_EXPORT void
 preview_reset_clicked_cb(GtkWidget *toggle, signal_user_data_t *ud)
 {
-    g_debug("preview_reset_clicked_cb()");
+    ghb_log_func();
     if (ud->preview->width <= 0 || ud->preview->height <= 0)
     {
         return;
@@ -1159,19 +1222,20 @@ preview_frame_value_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 G_MODULE_EXPORT gboolean
 preview_window_delete_cb(
     GtkWidget *widget,
+#if !GTK_CHECK_VERSION(4, 4, 0)
     GdkEvent *event,
+#endif
     signal_user_data_t *ud)
 {
     live_preview_stop(ud);
-    widget = GHB_WIDGET(ud->builder, "show_preview");
-    gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(widget), FALSE);
+    gtk_widget_set_visible(widget, FALSE);
     return TRUE;
 }
 
 G_MODULE_EXPORT void
 preview_duration_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
-    g_debug("preview_duration_changed_cb ()");
+    ghb_log_func();
     ghb_live_reset(ud);
     ghb_widget_to_setting (ud->prefs, widget);
     ghb_check_dependency(ud, widget, NULL);
@@ -1181,26 +1245,38 @@ preview_duration_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 
 static guint hud_timeout_id = 0;
 
+static gboolean in_hud = FALSE;
+
 static gboolean
 hud_timeout(signal_user_data_t *ud)
 {
     GtkWidget *widget;
 
-    g_debug("hud_timeout()");
+    ghb_log_func();
     widget = GHB_WIDGET(ud->builder, "preview_hud");
     gtk_widget_hide(widget);
     hud_timeout_id = 0;
     return FALSE;
 }
 
-static gboolean in_hud = FALSE;
-
+#if GTK_CHECK_VERSION(4, 4, 0)
+G_MODULE_EXPORT void
+hud_enter_cb(
+    GtkEventControllerMotion * econ,
+    double                     x,
+    double                     y,
+    GdkCrossingMode            cross,
+    GdkNotifyType              notify,
+    signal_user_data_t *ud)
+#else
 G_MODULE_EXPORT gboolean
 hud_enter_cb(
     GtkWidget *widget,
     GdkEventCrossing *event,
     signal_user_data_t *ud)
+#endif
 {
+    GtkWidget * hud;
     if (hud_timeout_id != 0)
     {
         GMainContext *mc;
@@ -1211,31 +1287,64 @@ hud_enter_cb(
         if (source != NULL)
             g_source_destroy(source);
     }
-    widget = GHB_WIDGET(ud->builder, "preview_hud");
-    if (!gtk_widget_get_visible(widget))
+    hud = GHB_WIDGET(ud->builder, "preview_hud");
+    if (!gtk_widget_get_visible(hud))
     {
-        gtk_widget_show(widget);
+        gtk_widget_show(hud);
     }
     hud_timeout_id = 0;
     in_hud = TRUE;
+#if !GTK_CHECK_VERSION(4, 4, 0)
     return FALSE;
+#endif
 }
 
+#if GTK_CHECK_VERSION(4, 4, 0)
+G_MODULE_EXPORT void
+hud_leave_cb(
+    GtkEventControllerMotion * econ,
+    GdkCrossingMode            cross,
+    GdkNotifyType              notify,
+    signal_user_data_t *ud)
+#else
 G_MODULE_EXPORT gboolean
 hud_leave_cb(
     GtkWidget *widget,
     GdkEventCrossing *event,
     signal_user_data_t *ud)
+#endif
 {
     in_hud = FALSE;
+#if !GTK_CHECK_VERSION(4, 4, 0)
     return FALSE;
+#endif
 }
 
+G_MODULE_EXPORT void
+preview_click_cb (GtkGesture *gest,
+                  gint n_press,
+                  gdouble x,
+                  gdouble y,
+                  signal_user_data_t *ud)
+{
+    if (n_press == 2)
+        g_action_activate(GHB_ACTION(ud->builder, "preview-fullscreen"), NULL);
+}
+
+#if GTK_CHECK_VERSION(4, 4, 0)
+G_MODULE_EXPORT void
+preview_leave_cb(
+    GtkEventControllerMotion * econ,
+    GdkCrossingMode            cross,
+    GdkNotifyType              notify,
+    signal_user_data_t *ud)
+#else
 G_MODULE_EXPORT gboolean
 preview_leave_cb(
     GtkWidget *widget,
     GdkEventCrossing *event,
     signal_user_data_t *ud)
+#endif
 {
     if (hud_timeout_id != 0)
     {
@@ -1248,15 +1357,28 @@ preview_leave_cb(
             g_source_destroy(source);
     }
     hud_timeout_id = g_timeout_add(300, (GSourceFunc)hud_timeout, ud);
+#if !GTK_CHECK_VERSION(4, 4, 0)
     return FALSE;
+#endif
 }
 
+#if GTK_CHECK_VERSION(4, 4, 0)
+G_MODULE_EXPORT void
+preview_motion_cb(
+    GtkEventControllerMotion * econ,
+    gdouble                    x,
+    gdouble                    y,
+    signal_user_data_t *ud)
+#else
 G_MODULE_EXPORT gboolean
 preview_motion_cb(
     GtkWidget *widget,
     GdkEventMotion *event,
     signal_user_data_t *ud)
+#endif
 {
+    GtkWidget * hud;
+
     if (hud_timeout_id != 0)
     {
         GMainContext *mc;
@@ -1267,28 +1389,39 @@ preview_motion_cb(
         if (source != NULL)
             g_source_destroy(source);
     }
-    widget = GHB_WIDGET(ud->builder, "preview_hud");
-    if (!gtk_widget_get_visible(widget))
+    hud = GHB_WIDGET(ud->builder, "preview_hud");
+    if (!gtk_widget_get_visible(hud))
     {
-        gtk_widget_show(widget);
+        gtk_widget_show(hud);
     }
     if (!in_hud)
     {
         hud_timeout_id = g_timeout_add_seconds(4, (GSourceFunc)hud_timeout, ud);
     }
+#if !GTK_CHECK_VERSION(4, 4, 0)
     return FALSE;
+#endif
 }
 
+// TODO: GTK4 eliminated "configure-event" signal.
+// From a read of the gtk4 code, it appears GDK_CONFIGURE does
+// not get propagated.  And there is no way to set position of
+// a window :*(
+//
+// Hopefully they will fix this or provide a better alternative
+// before gtk4 is released.
+#if !GTK_CHECK_VERSION(4, 4, 0)
 G_MODULE_EXPORT gboolean
 preview_configure_cb(
     GtkWidget *widget,
     GdkEventConfigure *event,
     signal_user_data_t *ud)
 {
-    gint x, y;
-
     if (gtk_widget_get_visible(widget))
     {
+        // TODO: can this be done in GTK4?
+        gint x, y;
+
         gtk_window_get_position(GTK_WINDOW(widget), &x, &y);
         ghb_dict_set_int(ud->prefs, "preview_x", x);
         ghb_dict_set_int(ud->prefs, "preview_y", y);
@@ -1298,43 +1431,57 @@ preview_configure_cb(
     }
     return FALSE;
 }
+#endif
 
+#if !GTK_CHECK_VERSION(4, 4, 0)
+// GTK4 no longer has GDK_WINDOW_STATE events :*(
 G_MODULE_EXPORT gboolean
 preview_state_cb(
     GtkWidget *widget,
     GdkEvent  *event,
     signal_user_data_t *ud)
 {
-    GdkEventWindowState * wse = (GdkEventWindowState*)event;
-    if (wse->type == GDK_WINDOW_STATE)
+    GdkEventType type = ghb_event_get_event_type(event);
+    if (type == GDK_WINDOW_STATE)
     {
         // Look for transition to iconified state.
         // Toggle "Show Preview" button when iconified.
         // I only do this because there seems to be no
-        // way to reliably disable the iconfy button without
+        // way to reliably disable the iconify button without
         // also disabling the maximize button.
+        GdkEventWindowState * wse = (GdkEventWindowState*)event;
         if (wse->changed_mask & wse->new_window_state &
             GDK_WINDOW_STATE_ICONIFIED)
         {
             live_preview_stop(ud);
-            GtkWidget *widget = GHB_WIDGET(ud->builder, "show_preview");
-            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(widget), FALSE);
         }
+        ud->preview->is_fullscreen = wse->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
     }
     return FALSE;
 }
+#endif
 
 G_MODULE_EXPORT void
 preview_resize_cb(
     GtkWidget     *widget,
+#if GTK_CHECK_VERSION(4, 4, 0)
+    int width,
+    int height,
+    int baseline,
+#else
     GdkRectangle  *rect,
+#endif
     signal_user_data_t *ud)
 {
-    if (ud->preview->render_width != rect->width ||
-        ud->preview->render_height != rect->height)
+#if !GTK_CHECK_VERSION(4, 4, 0)
+    int width  = rect->width;
+    int height = rect->height;
+#endif
+    if (ud->preview->render_width != width ||
+        ud->preview->render_height != height)
     {
-        ud->preview->render_width  = rect->width;
-        ud->preview->render_height = rect->height;
+        ud->preview->render_width  = width;
+        ud->preview->render_height = height;
         if (ud->preview->scaled_pix != NULL)
             g_object_unref(ud->preview->scaled_pix);
         ud->preview->scaled_pix = do_preview_scaling(ud, ud->preview->pix);
@@ -1348,7 +1495,7 @@ preview_resize_cb(
             GtkWidget * widget = GHB_WIDGET(ud->builder, "preview_reset");
             gtk_widget_hide(widget);
         }
-        else
+        else if (!ud->preview->is_fullscreen)
         {
             GtkWidget * widget = GHB_WIDGET(ud->builder, "preview_reset");
             gtk_widget_show(widget);
@@ -1359,13 +1506,16 @@ preview_resize_cb(
 G_MODULE_EXPORT void
 show_crop_changed_cb(GtkWidget *widget, signal_user_data_t *ud)
 {
-    g_debug("show_crop_changed_cb ()");
+#if 0
+    // Disabled until we reimplement this or come up with something better
+    //
+    ghb_log_func();
     ghb_widget_to_setting(ud->prefs, widget);
     ghb_check_dependency(ud, widget, NULL);
     ghb_live_reset(ud);
     if (gtk_widget_is_sensitive(widget))
-        ghb_set_scale(ud, GHB_PIC_KEEP_PAR);
+        ghb_set_scale(ud, 0);
     ghb_pref_save(ud->prefs, "preview_show_crop");
     ghb_rescale_preview_image(ud);
+#endif
 }
-
