@@ -1,14 +1,13 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * settings.c
- * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2023 <stebbins@stebbins>
  *
  * settings.c is free software.
  *
  * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ * GNU General Public License version 2, as published by the Free Software
+ * Foundation.
  *
  * settings.c is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,12 +21,13 @@
  *  Boston, MA  02110-1301, USA.
  */
 
+#include "ghbcompat.h"
+
 #include <fcntl.h>
 #include <unistd.h>
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <string.h>
-#include "ghbcompat.h"
 #include "settings.h"
 #include "hb-backend.h"
 #include "values.h"
@@ -38,7 +38,7 @@ void ghb_pref_audio_init(signal_user_data_t *ud);
 GObject*
 debug_get_object(GtkBuilder* b, const gchar *n)
 {
-    g_message("name %s\n", n);
+    g_debug("name %s\n", n);
     return gtk_builder_get_object(b, n);
 }
 
@@ -60,220 +60,6 @@ ghb_settings_combo_option(const GhbValue *settings, const gchar *key)
     return ghb_lookup_combo_option(key, ghb_dict_get_value(settings, key));
 }
 
-GhbValue *ghb_get_title_settings(GhbValue *settings)
-{
-    GhbValue *title;
-    title = ghb_dict_get(settings, "Title");
-    return title;
-}
-
-GhbValue *ghb_get_title_audio_list(GhbValue *settings)
-{
-    GhbValue *title_dict = ghb_get_title_settings(settings);
-    GhbValue *audio_list = ghb_dict_get(title_dict, "AudioList");
-    return audio_list;
-}
-
-GhbValue *ghb_get_title_subtitle_list(GhbValue *settings)
-{
-    GhbValue *title_dict = ghb_get_title_settings(settings);
-    GhbValue *subtitle_list = ghb_dict_get(title_dict, "SubtitleList");
-    return subtitle_list;
-}
-
-GhbValue *ghb_get_job_settings(GhbValue *settings)
-{
-    GhbValue *job;
-    job = ghb_dict_get(settings, "Job");
-    if (job == NULL)
-    {
-        job = ghb_dict_new();
-        ghb_dict_set(settings, "Job", job);
-    }
-    return job;
-}
-
-GhbValue* ghb_get_job_dest_settings(GhbValue *settings)
-{
-    GhbValue *job  = ghb_get_job_settings(settings);
-    GhbValue *dest = ghb_dict_get(job, "Destination");
-    if (dest == NULL)
-    {
-        dest = ghb_dict_new();
-        ghb_dict_set(job, "Destination", dest);
-    }
-    return dest;
-}
-
-GhbValue* ghb_get_job_chapter_list(GhbValue *settings)
-{
-    GhbValue *dest     = ghb_get_job_dest_settings(settings);
-    GhbValue *chapters = ghb_dict_get(dest, "ChapterList");
-    if (chapters == NULL)
-    {
-        chapters = ghb_array_new();
-        ghb_dict_set(dest, "ChapterList", chapters);
-    }
-    return chapters;
-}
-
-GhbValue* ghb_get_job_mp4_settings(GhbValue *settings)
-{
-    GhbValue *dest = ghb_get_job_dest_settings(settings);
-    GhbValue *mp4  = ghb_dict_get(dest, "Mp4Options");
-    if (mp4 == NULL)
-    {
-        mp4 = ghb_dict_new();
-        ghb_dict_set(dest, "Mp4Options", mp4);
-    }
-    return mp4;
-}
-
-GhbValue* ghb_get_job_source_settings(GhbValue *settings)
-{
-    GhbValue *job    = ghb_get_job_settings(settings);
-    GhbValue *source = ghb_dict_get(job, "Source");
-    if (source == NULL)
-    {
-        source = ghb_dict_new();
-        ghb_dict_set(job, "Source", source);
-    }
-    return source;
-}
-
-GhbValue* ghb_get_job_range_settings(GhbValue *settings)
-{
-    GhbValue *source = ghb_get_job_source_settings(settings);
-    GhbValue *range  = ghb_dict_get(source, "Range");
-    if (range == NULL)
-    {
-        range = ghb_dict_new();
-        ghb_dict_set(source, "Range", range);
-    }
-    return range;
-}
-
-GhbValue* ghb_get_job_par_settings(GhbValue *settings)
-{
-    GhbValue *job = ghb_get_job_settings(settings);
-    GhbValue *par = ghb_dict_get(job, "PAR");
-    if (par == NULL)
-    {
-        par = ghb_dict_new();
-        ghb_dict_set(job, "PAR", par);
-    }
-    return par;
-}
-
-GhbValue* ghb_get_job_video_settings(GhbValue *settings)
-{
-    GhbValue *job   = ghb_get_job_settings(settings);
-    GhbValue *video = ghb_dict_get(job, "Video");
-    if (video == NULL)
-    {
-        video = ghb_dict_new();
-        ghb_dict_set(job, "Video", video);
-    }
-    return video;
-}
-
-GhbValue *ghb_get_job_audio_settings(GhbValue *settings)
-{
-    GhbValue *job   = ghb_get_job_settings(settings);
-    GhbValue *audio = ghb_dict_get(job, "Audio");
-    if (audio == NULL)
-    {
-        audio = ghb_dict_new();
-        ghb_dict_set(job, "Audio", audio);
-    }
-    return audio;
-}
-
-GhbValue *ghb_get_job_audio_list(GhbValue *settings)
-{
-    GhbValue *audio_dict = ghb_get_job_audio_settings(settings);
-    GhbValue *audio_list = ghb_dict_get(audio_dict, "AudioList");
-    if (audio_list == NULL)
-    {
-        audio_list = ghb_array_new();
-        ghb_dict_set(audio_dict, "AudioList", audio_list);
-    }
-    return audio_list;
-}
-
-GhbValue *ghb_get_job_subtitle_settings(GhbValue *settings)
-{
-    GhbValue *job = ghb_get_job_settings(settings);
-    GhbValue *sub = ghb_dict_get(job, "Subtitle");
-    if (sub == NULL)
-    {
-        sub = ghb_dict_new();
-        ghb_dict_set(job, "Subtitle", sub);
-    }
-    return sub;
-}
-
-GhbValue *ghb_get_job_subtitle_list(GhbValue *settings)
-{
-    GhbValue *sub_dict = ghb_get_job_subtitle_settings(settings);
-    GhbValue *sub_list = ghb_dict_get(sub_dict, "SubtitleList");
-    if (sub_list == NULL)
-    {
-        sub_list = ghb_array_new();
-        ghb_dict_set(sub_dict, "SubtitleList", sub_list);
-    }
-    return sub_list;
-}
-
-GhbValue *ghb_get_job_subtitle_search(GhbValue *settings)
-{
-    GhbValue *sub_dict   = ghb_get_job_subtitle_settings(settings);
-    GhbValue *sub_search = ghb_dict_get(sub_dict, "Search");
-    if (sub_search == NULL)
-    {
-        sub_search = ghb_dict_new();
-        ghb_dict_set(sub_dict, "Search", sub_search);
-        ghb_dict_set_bool(sub_search, "Enable", 0);
-    }
-    return sub_search;
-}
-
-GhbValue* ghb_get_job_metadata_settings(GhbValue *settings)
-{
-    GhbValue *job  = ghb_get_job_settings(settings);
-    GhbValue *meta = ghb_dict_get(job, "Metadata");
-    if (meta == NULL)
-    {
-        meta = ghb_dict_new();
-        ghb_dict_set(job, "Metadata", meta);
-    }
-    return meta;
-}
-
-GhbValue* ghb_get_job_filter_settings(GhbValue *settings)
-{
-    GhbValue *job    = ghb_get_job_settings(settings);
-    GhbValue *filter = ghb_dict_get(job, "Filters");
-    if (filter == NULL)
-    {
-        filter = ghb_dict_new();
-        ghb_dict_set(job, "Filters", filter);
-    }
-    return filter;
-}
-
-GhbValue* ghb_get_job_filter_list(GhbValue *settings)
-{
-    GhbValue *filter = ghb_get_job_filter_settings(settings);
-    GhbValue *list   = ghb_dict_get(filter, "FilterList");
-    if (list == NULL)
-    {
-        list = ghb_dict_new();
-        ghb_dict_set(filter, "FilterList", list);
-    }
-    return list;
-}
-
 // Map widget names to setting keys
 // Widgets that map to settings have names
 // of this format: s_<setting key>
@@ -282,8 +68,9 @@ ghb_get_setting_key(GtkWidget *widget)
 {
     const gchar *name;
 
-    g_debug("get_setting_key ()\n");
+    ghb_log_func();
     if (widget == NULL) return NULL;
+    g_return_val_if_fail(GTK_IS_WIDGET(widget), NULL);
     name = gtk_buildable_get_name(GTK_BUILDABLE(widget));
 
     if (name == NULL || !strncmp(name, "Gtk", 3))
@@ -293,7 +80,7 @@ ghb_get_setting_key(GtkWidget *widget)
     if (name == NULL)
     {
         // Bad widget pointer?  Should never happen.
-        g_debug("Bad widget\n");
+        g_debug("Bad widget");
         return NULL;
     }
     return name;
@@ -308,7 +95,7 @@ ghb_widget_value(GtkWidget *widget)
 
     if (widget == NULL)
     {
-        g_debug("NULL widget\n");
+        g_debug("NULL widget");
         return NULL;
     }
 
@@ -316,18 +103,20 @@ ghb_widget_value(GtkWidget *widget)
     name = ghb_get_setting_key(widget);
     if (type == GTK_TYPE_ENTRY)
     {
-        const gchar *str = gtk_entry_get_text(GTK_ENTRY(widget));
+        const gchar *str = ghb_editable_get_text(widget);
         value = ghb_string_value_new(str);
     }
     else if (type == GTK_TYPE_RADIO_BUTTON)
     {
         gboolean bval;
+#if !GTK_CHECK_VERSION(4, 4, 0)
         bval = gtk_toggle_button_get_inconsistent(GTK_TOGGLE_BUTTON(widget));
         if (bval)
         {
             value = ghb_bool_value_new(FALSE);
         }
         else
+#endif
         {
             bval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
             value = ghb_bool_value_new(bval);
@@ -373,7 +162,7 @@ ghb_widget_value(GtkWidget *widget)
         else if (gtk_combo_box_get_has_entry(GTK_COMBO_BOX(widget)))
         {
             const gchar *str;
-            str = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(widget))));
+            str = ghb_editable_get_text(gtk_bin_get_child(GTK_BIN(widget)));
             if (str == NULL) str = "";
             value = ghb_string_value_new(str);
         }
@@ -517,7 +306,7 @@ ghb_widget_to_setting(GhbValue *settings, GtkWidget *widget)
     GhbValue *value;
 
     if (widget == NULL) return;
-    g_debug("ghb_widget_to_setting");
+    ghb_log_func();
     // Find corresponding setting
     key = ghb_get_setting_key(widget);
     if (key == NULL) return;
@@ -528,7 +317,7 @@ ghb_widget_to_setting(GhbValue *settings, GtkWidget *widget)
     }
     else
     {
-        g_debug("No value found for %s\n", key);
+        g_debug("No value found for %s", key);
     }
 }
 
@@ -555,7 +344,7 @@ ghb_update_widget(GtkWidget *widget, const GhbValue *value)
 
     if (type == GTK_TYPE_ENTRY)
     {
-        gtk_entry_set_text((GtkEntry*)widget, str);
+        ghb_editable_set_text(widget, str);
     }
     else if (type == GTK_TYPE_RADIO_BUTTON)
     {
@@ -624,7 +413,7 @@ ghb_update_widget(GtkWidget *widget, const GhbValue *value)
                 GtkEntry *entry = GTK_ENTRY(gtk_bin_get_child(GTK_BIN(widget)));
                 if (entry)
                 {
-                    gtk_entry_set_text (entry, str);
+                    ghb_editable_set_text(entry, str);
                 }
                 else
                 {
@@ -697,11 +486,12 @@ ghb_update_widget(GtkWidget *widget, const GhbValue *value)
             }
             else
             {
-                gchar *dirname;
+                gchar * dirname;
 
                 dirname = g_path_get_dirname(str);
                 gtk_file_chooser_set_current_folder(
                     GTK_FILE_CHOOSER(widget), dirname);
+                gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(widget));
                 g_free(dirname);
             }
         }
@@ -719,7 +509,7 @@ ghb_ui_update_from_settings(signal_user_data_t *ud, const gchar *name, const Ghb
     GObject *object;
     GhbValue * value;
 
-    g_debug("ghb_ui_update_from_settings() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL)
         return 0;
     value = ghb_dict_get_value(settings, name);
@@ -728,7 +518,7 @@ ghb_ui_update_from_settings(signal_user_data_t *ud, const gchar *name, const Ghb
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
@@ -743,13 +533,13 @@ ghb_ui_update(signal_user_data_t *ud, const gchar *name, const GhbValue *value)
 {
     GObject *object;
 
-    g_debug("ghb_ui_update() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL || value == NULL)
         return 0;
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
@@ -768,13 +558,13 @@ ghb_ui_settings_update(
 {
     GObject *object;
 
-    g_debug("ghb_ui_update() %s", name);
+    ghb_log_func_str(name);
     if (name == NULL || value == NULL)
         return 0;
     object = GHB_OBJECT(ud->builder, name);
     if (object == NULL)
     {
-        g_debug("Failed to find widget for key: %s\n", name);
+        g_debug("Failed to find widget for key: %s", name);
         return -1;
     }
     ghb_update_widget((GtkWidget*)object, value);
@@ -783,4 +573,3 @@ ghb_ui_settings_update(
     ghb_widget_to_setting(settings, (GtkWidget*)object);
     return 0;
 }
-
