@@ -1,19 +1,28 @@
 /* fifo.c
 
-   Copyright (c) 2003-2018 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
+   Copyright 2022 NVIDIA Corporation
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
-#include "hb.h"
-#ifdef USE_QSV
-#include "qsv_libav.h"
+#include "libavcodec/avcodec.h"
+
+#include "handbrake/handbrake.h"
+#if HB_PROJECT_FEATURE_QSV
+#include "handbrake/qsv_libav.h"
+#include "handbrake/qsv_common.h"
+#endif
+
+#ifdef __APPLE__
+#include <CoreMedia/CoreMedia.h>
+#include "platform/macosx/vt_common.h"
 #endif
 
 #ifndef SYS_DARWIN
-#if defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
+#if defined( SYS_FREEBSD ) || defined( SYS_NETBSD ) || defined( SYS_OPENBSD )
 #include <stdlib.h>
 #else
 #include <malloc.h>
@@ -55,7 +64,7 @@ struct hb_fifo_s
 };
 
 #if defined(HB_FIFO_DEBUG)
-static hb_fifo_t fifo_list = 
+static hb_fifo_t fifo_list =
 {
     .next = NULL
 };
@@ -106,6 +115,10 @@ void hb_buffer_pool_init( void )
      * 2^25 will get passed through to malloc. */
     int i;
 
+    // Create a queue with empty buffers for non native storage types
+    buffers.pool[0] = hb_fifo_init(BUFFER_POOL_MAX_ELEMENTS*10, 1);
+    buffers.pool[0]->buffer_size = 0;
+
     // Create larger queue for 2^10 bucket since all allocations smaller than
     // 2^10 come from here.
     buffers.pool[BUFFER_POOL_FIRST] = hb_fifo_init(BUFFER_POOL_MAX_ELEMENTS*10, 1);
@@ -295,7 +308,7 @@ void hb_buffer_pool_free( void )
             if( b->data )
             {
                 freed += b->alloc;
-                free(b->data);
+                av_free(b->data);
             }
             free( b );
             count++;
@@ -328,6 +341,11 @@ void hb_buffer_pool_free( void )
 static hb_fifo_t *size_to_pool( int size )
 {
 #if !defined(HB_NO_BUFFER_POOL)
+    if (size == 0)
+    {
+        return buffers.pool[0];
+    }
+
     int i;
     for ( i = BUFFER_POOL_FIRST; i <= BUFFER_POOL_LAST; ++i )
     {
@@ -344,11 +362,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
 {
     hb_buffer_t * b;
     // Certain libraries (hrm ffmpeg) expect buffers passed to them to
-    // end on certain alignments (ffmpeg is 8). So allocate some extra bytes.
+    // end on certain alignments. So allocate some extra bytes.
     // Note that we can't simply align the end of our buffer because
     // sometimes we feed data to these libraries starting from arbitrary
     // points within the buffer.
-    int alloc = size + 16;
+    int alloc = size ? size + AV_INPUT_BUFFER_PADDING_SIZE : 0;
     hb_fifo_t *buffer_pool = size_to_pool( alloc );
 
     if( buffer_pool )
@@ -366,7 +384,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
             memset( b, 0, sizeof(hb_buffer_t) );
             b->alloc          = buffer_pool->buffer_size;
             b->size           = size;
-            b->data           = data;
+            if (size)
+            {
+                b->data           = data;
+            }
+            b->storage        = NULL;
             b->s.start        = AV_NOPTS_VALUE;
             b->s.stop         = AV_NOPTS_VALUE;
             b->s.renderOffset = AV_NOPTS_VALUE;
@@ -386,7 +408,7 @@ hb_buffer_t * hb_buffer_init_internal( int size )
      */
     if( !( b = calloc( sizeof( hb_buffer_t ), 1 ) ) )
     {
-        hb_log( "out of memory" );
+        hb_error( "out of memory" );
         return NULL;
     }
 
@@ -395,18 +417,10 @@ hb_buffer_t * hb_buffer_init_internal( int size )
 
     if (size)
     {
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-        b->data  = malloc( b->alloc );
-#elif defined( SYS_CYGWIN )
-        /* FIXME */
-        b->data  = malloc( b->alloc + 17 );
-#else
-        b->data  = memalign( 16, b->alloc );
-#endif
-
+        b->data = av_malloc(b->alloc);
         if( !b->data )
         {
-            hb_log( "out of memory" );
+            hb_error( "out of memory" );
             free( b );
             return NULL;
         }
@@ -429,6 +443,11 @@ hb_buffer_t * hb_buffer_init_internal( int size )
     return b;
 }
 
+hb_buffer_t * hb_buffer_wrapper_init()
+{
+    return hb_buffer_init_internal(0);
+}
+
 hb_buffer_t * hb_buffer_init( int size )
 {
     return hb_buffer_init_internal(size);
@@ -445,13 +464,25 @@ void hb_buffer_realloc( hb_buffer_t * b, int size )
 {
     if ( size > b->alloc || b->data == NULL )
     {
-        uint32_t orig = b->data != NULL ? b->alloc : 0;
-        hb_fifo_t *buffer_pool = size_to_pool(size);
+        uint8_t   * tmp;
+        uint32_t    orig = b->data != NULL ? b->alloc : 0;
+        hb_fifo_t * buffer_pool = size_to_pool(size);
+
         if (buffer_pool != NULL)
         {
             size = buffer_pool->buffer_size;
         }
-        b->data  = realloc( b->data, size );
+        tmp = av_malloc(size);
+        if (tmp == NULL)
+        {
+            return;
+        }
+        if (b->data != NULL)
+        {
+            memcpy(tmp, b->data, b->alloc);
+            av_free(b->data);
+        }
+        b->data  = tmp;
         b->alloc = size;
 
         hb_lock(buffers.lock);
@@ -462,38 +493,255 @@ void hb_buffer_realloc( hb_buffer_t * b, int size )
 
 void hb_buffer_reduce( hb_buffer_t * b, int size )
 {
-
-    if ( size < b->alloc / 8 || b->data == NULL )
+    if (b->storage_type == STANDARD && (size < b->alloc / 8 || b->data == NULL))
     {
-        hb_buffer_t * tmp = hb_buffer_init( size );
-
-        hb_buffer_swap_copy( b, tmp );
-        memcpy( b->data, tmp->data, size );
-        tmp->next = NULL;
-        hb_buffer_close( &tmp );
+        hb_buffer_t *tmp = hb_buffer_init(size);
+        if (tmp)
+        {
+            hb_buffer_swap_copy(b, tmp);
+            if (tmp->data)
+            {
+                memcpy(b->data, tmp->data, size);
+            }
+            tmp->next = NULL;
+        }
+        hb_buffer_close(&tmp);
     }
 }
 
-hb_buffer_t * hb_buffer_dup( const hb_buffer_t * src )
+AVFrameSideData *hb_buffer_new_side_data_from_buf(hb_buffer_t *buf,
+                                                  enum AVFrameSideDataType type,
+                                                  AVBufferRef *side_data_buf)
 {
-
-    hb_buffer_t * buf;
-
-    if ( src == NULL )
-        return NULL;
-
-    buf = hb_buffer_init( src->size );
-    if ( buf )
+    AVFrameSideData *ret;
+    if (buf->storage_type == AVFRAME)
     {
-        memcpy( buf->data, src->data, src->size );
-        buf->s = src->s;
-        buf->f = src->f;
-        if ( buf->s.type == FRAME_BUF )
-            hb_buffer_init_planes( buf );
+        AVFrame *frame = (AVFrame *)buf->storage;
+        ret = av_frame_new_side_data_from_buf(frame, type, side_data_buf);
+        buf->side_data = (void **)frame->side_data;
+        buf->nb_side_data = frame->nb_side_data;
+        return ret;
+    }
+    else
+    {
+        AVFrameSideData **tmp;
+
+        if (!buf)
+        {
+            return NULL;
+        }
+
+        if (buf->nb_side_data > INT_MAX / sizeof(*buf->side_data) - 1)
+        {
+            return NULL;
+        }
+
+        tmp = av_realloc(buf->side_data, (buf->nb_side_data + 1) * sizeof(*buf->side_data));
+        if (!tmp)
+        {
+            return NULL;
+        }
+        buf->side_data = (void **)tmp;
+
+        ret = av_mallocz(sizeof(*ret));
+        if (!ret)
+        {
+            return NULL;
+        }
+
+        ret->buf = side_data_buf;
+        ret->data = ret->buf->data;
+        ret->size = side_data_buf->size;
+        ret->type = type;
+
+        buf->side_data[buf->nb_side_data++] = ret;
+
+        return ret;
+    }
+}
+
+static void free_side_data(AVFrameSideData **ptr_sd)
+{
+    AVFrameSideData *sd = *ptr_sd;
+
+    av_buffer_unref(&sd->buf);
+    av_dict_free(&sd->metadata);
+    av_freep(ptr_sd);
+}
+
+void hb_buffer_remove_side_data(hb_buffer_t *buf, enum AVFrameSideDataType type)
+{
+    if (buf->storage_type == AVFRAME)
+    {
+        AVFrame *frame = (AVFrame *)buf->storage;
+        av_frame_remove_side_data(frame, type);
+        buf->nb_side_data = frame->nb_side_data;
+    }
+    else
+    {
+        for (int i = buf->nb_side_data - 1; i >= 0; i--)
+        {
+            AVFrameSideData *sd = buf->side_data[i];
+            if (sd->type == type)
+            {
+                free_side_data((AVFrameSideData **)&buf->side_data[i]);
+                buf->side_data[i] = buf->side_data[buf->nb_side_data - 1];
+                buf->nb_side_data--;
+            }
+        }
+    }
+}
+
+void hb_buffer_wipe_side_data(hb_buffer_t *buf)
+{
+    for (int i = 0; i < buf->nb_side_data; i++)
+    {
+        free_side_data((AVFrameSideData **)&buf->side_data[i]);
+    }
+    buf->nb_side_data = 0;
+
+    av_freep(&buf->side_data);
+}
+
+void hb_buffer_copy_side_data(hb_buffer_t *dst, const hb_buffer_t *src)
+{
+    for (int i = 0; i < src->nb_side_data; i++)
+    {
+        const AVFrameSideData *sd_src = src->side_data[i];
+        AVBufferRef *ref = av_buffer_ref(sd_src->buf);
+        AVFrameSideData *sd_dst = hb_buffer_new_side_data_from_buf(dst, sd_src->type, ref);
+        if (!sd_dst)
+        {
+            av_buffer_unref(&ref);
+            hb_buffer_wipe_side_data(dst);
+        }
+    }
+}
+
+void hb_buffer_copy_props(hb_buffer_t *dst, const hb_buffer_t *src)
+{
+    dst->s = src->s;
+    hb_buffer_copy_side_data(dst, src);
+}
+
+static int copy_hwframe_to_video_buffer(const AVFrame *frame, hb_buffer_t *buf)
+{
+    int ret;
+    AVFrame *hw_frame = av_frame_alloc();
+
+    ret = av_frame_copy_props(hw_frame, frame);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_frame_copy_props");
+    }
+    ret = av_hwframe_get_buffer(frame->hw_frames_ctx, hw_frame, 0);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_hwframe_get_buffer failed");
+    }
+    ret = av_hwframe_transfer_data(hw_frame, frame, 0);
+    if (ret < 0)
+    {
+        hb_log("fifo: av_hwframe_transfer_data failed");
     }
 
-#ifdef USE_QSV
-    memcpy(&buf->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
+    buf->storage = hw_frame;
+    buf->storage_type = AVFRAME;
+
+    return ret;
+}
+
+static void copy_avframe_to_video_buffer(const AVFrame *frame, hb_buffer_t *buf)
+{
+    for (int pp = 0; pp <= buf->f.max_plane; pp++)
+    {
+        if (buf->plane[pp].stride == frame->linesize[pp])
+        {
+            memcpy(buf->plane[pp].data, frame->data[pp], frame->linesize[pp] * buf->plane[pp].height);
+        }
+        else
+        {
+            const int width     = buf->plane[pp].width;
+            const int height    = buf->plane[pp].height;
+            const int stride    = buf->plane[pp].stride;
+            const int linesize  = frame->linesize[pp];
+            uint8_t *dst = buf->plane[pp].data;
+            uint8_t *src = frame->data[pp];
+            for (int yy = 0; yy < height; yy++)
+            {
+                memcpy(dst, src, width);
+                dst += stride;
+                src += linesize;
+            }
+        }
+    }
+}
+
+hb_buffer_t * hb_buffer_dup(const hb_buffer_t *src)
+{
+    hb_buffer_t *buf = NULL;
+
+    if (src == NULL)
+    {
+        return NULL;
+    }
+
+    if (src->storage_type == STANDARD)
+    {
+        buf = hb_buffer_init(src->size);
+        if (buf)
+        {
+            buf->f = src->f;
+            hb_buffer_copy_props(buf, src);
+
+            if (buf->s.type == FRAME_BUF)
+            {
+                hb_buffer_init_planes(buf);
+            }
+
+            memcpy(buf->data, src->data, src->size);
+        }
+    }
+    else if (src->storage_type == AVFRAME)
+    {
+        const AVFrame *frame = (AVFrame *)src->storage;
+
+        // If it's an hardware frame, make a copy
+        // into another hardware AVFrame.
+        if (frame->hw_frames_ctx)
+        {
+            buf = hb_buffer_wrapper_init();
+            if (buf)
+            {
+                buf->f = src->f;
+                hb_buffer_copy_props(buf, src);
+                copy_hwframe_to_video_buffer(frame, buf);
+            }
+        }
+        // If not, copy the content to a standard hb_buffer
+        else
+        {
+            buf = hb_frame_buffer_init(src->f.fmt, src->f.width, src->f.height);
+            if (buf)
+            {
+                buf->f = src->f;
+                hb_buffer_copy_props(buf, src);
+                copy_avframe_to_video_buffer(frame, buf);
+            }
+        }
+    }
+#ifdef __APPLE__
+    else if (src->storage_type == COREMEDIA)
+    {
+        buf = hb_vt_buffer_dup(src);
+    }
+#endif
+
+#if HB_PROJECT_FEATURE_QSV
+    if (buf)
+    {
+        memcpy(&buf->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
+    }
 #endif
 
     return buf;
@@ -508,78 +756,60 @@ int hb_buffer_copy(hb_buffer_t * dst, const hb_buffer_t * src)
         return -1;
 
     memcpy( dst->data, src->data, src->size );
-    dst->s = src->s;
     dst->f = src->f;
+    hb_buffer_copy_props(dst, src);
     if (dst->s.type == FRAME_BUF)
         hb_buffer_init_planes(dst);
 
     return 0;
 }
 
-static void hb_buffer_init_planes_internal( hb_buffer_t * b, uint8_t * has_plane )
+void hb_buffer_init_planes(hb_buffer_t * b)
 {
-    uint8_t * plane = b->data;
-    int p;
+    uint8_t * data = b->data;
+    int       pp;
 
-    for( p = 0; p < 4; p++ )
+    for( pp = 0; pp <= b->f.max_plane; pp++ )
     {
-        if ( has_plane[p] )
-        {
-            b->plane[p].data = plane;
-            b->plane[p].stride = hb_image_stride( b->f.fmt, b->f.width, p );
-            b->plane[p].height_stride = hb_image_height_stride( b->f.fmt, b->f.height, p );
-            b->plane[p].width  = hb_image_width( b->f.fmt, b->f.width, p );
-            b->plane[p].height = hb_image_height( b->f.fmt, b->f.height, p );
-            b->plane[p].size   = b->plane[p].stride * b->plane[p].height_stride;
-            plane += b->plane[p].size;
-        }
+        b->plane[pp].data = data;
+        b->plane[pp].stride        = hb_image_stride(b->f.fmt, b->f.width, pp);
+        b->plane[pp].height_stride = hb_image_height_stride(b->f.fmt,
+                                                            b->f.height, pp);
+        b->plane[pp].width         = hb_image_width(b->f.fmt, b->f.width, pp);
+        b->plane[pp].height        = hb_image_height(b->f.fmt, b->f.height, pp);
+        b->plane[pp].size          = b->plane[pp].stride *
+                                     b->plane[pp].height_stride;
+        data                      += b->plane[pp].size;
     }
 }
 
-void hb_buffer_init_planes( hb_buffer_t * b )
-{
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(b->f.fmt);
-    int p;
-
-    if (desc == NULL)
-    {
-        return;
-    }
-
-    uint8_t has_plane[4] = {0,};
-
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
-    hb_buffer_init_planes_internal( b, has_plane );
-}
-
 // this routine gets a buffer for an uncompressed picture
 // with pixel format pix_fmt and dimensions width x height.
 hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
-    hb_buffer_t * buf;
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(pix_fmt);
+    hb_buffer_t              * buf;
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp, max_plane = 0;
 
     if (desc == NULL)
     {
         return NULL;
     }
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
 
     int size = 0;
-    for( p = 0; p < 4; p++ )
+    for (ii = 0; ii < desc->nb_components; ii++)
     {
-        if ( has_plane[p] )
+        pp    = desc->comp[ii].plane;
+        if (pp > max_plane)
         {
-            size += hb_image_stride( pix_fmt, width, p ) * 
-                    hb_image_height_stride( pix_fmt, height, p );
+            max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride( pix_fmt, width, pp ) *
+                    hb_image_height_stride( pix_fmt, height, pp );
         }
     }
 
@@ -588,39 +818,121 @@ hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
     if( buf == NULL )
         return NULL;
 
+    buf->f.max_plane = max_plane;
     buf->s.type = FRAME_BUF;
     buf->f.width = width;
     buf->f.height = height;
     buf->f.fmt = pix_fmt;
 
-    hb_buffer_init_planes_internal( buf, has_plane );
+    hb_buffer_init_planes(buf);
+
     return buf;
 }
 
+void hb_frame_buffer_blank_stride(hb_buffer_t * buf)
+{
+    uint8_t * data;
+    int       pp, yy, width, height, stride, height_stride;
+
+    for (pp = 0; pp <= buf->f.max_plane; pp++)
+    {
+        data          = buf->plane[pp].data;
+        width         = buf->plane[pp].width;
+        height        = buf->plane[pp].height;
+        stride        = buf->plane[pp].stride;
+        height_stride = buf->plane[pp].height_stride;
+
+        if (data != NULL)
+        {
+            // Blank right margin
+            for (yy = 0; yy < height; yy++)
+            {
+                memset(data + yy * stride + width, 0x80, stride - width);
+            }
+            // Blank bottom margin
+            for (yy = height; yy < height_stride; yy++)
+            {
+                memset(data + yy * stride, 0x80, stride);
+            }
+        }
+    }
+}
+
+void hb_frame_buffer_mirror_stride(hb_buffer_t * buf)
+{
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(buf->f.fmt);
+    uint8_t * data;
+    int       pp, ii, yy, width, height, stride, height_stride;
+    int       bps, pos, margin, margin_front, margin_back;
+
+    bps = desc->comp[0].depth > 8 ? 2 : 1;
+
+    for (pp = 0; pp <= buf->f.max_plane; pp++)
+    {
+        data          = buf->plane[pp].data;
+        width         = buf->plane[pp].width;
+        height        = buf->plane[pp].height;
+        stride        = buf->plane[pp].stride;
+        height_stride = buf->plane[pp].height_stride;
+        if (data != NULL)
+        {
+            margin       = stride / bps - width;
+            margin_front = margin / 2;
+            margin_back  = margin - margin_front;
+            width       *= bps;
+            for (yy = 0; yy < height; yy++)
+            {
+                // Mirror final row pixels into front of stride region
+                pos = yy * stride + width;
+                for (ii = 0; ii < margin_back; ii++)
+                {
+                    *(data + pos + ii) = *(data + pos - ii - 1);
+                }
+                // Mirror start of next row into end of stride region
+                pos = (yy + 1) * stride - 1;
+                for (ii = 0; ii < margin_front; ii++)
+                {
+                    *(data + pos - ii) = *(data + pos + ii + 1);
+                }
+            }
+            // Mirror bottom rows into height_stride
+            pos = height * stride;
+            for (ii = 0; ii < height_stride - height; ii++)
+            {
+                memcpy(data + pos + ii * stride,
+                       data + pos - ((ii + 1) * stride), stride);
+            }
+        }
+    }
+}
+
 // this routine reallocs a buffer for an uncompressed YUV420 video frame
 // with dimensions width x height.
 void hb_video_buffer_realloc( hb_buffer_t * buf, int width, int height )
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(buf->f.fmt);
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(buf->f.fmt);
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp;
 
     if (desc == NULL)
     {
         return;
     }
-    for( p = 0; p < 4; p++ )
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
 
+    buf->f.max_plane = 0;
     int size = 0;
-    for( p = 0; p < 4; p++ )
+    for (ii = 0; ii < desc->nb_components; ii++)
     {
-        if ( has_plane[p] )
+        pp = desc->comp[ii].plane;
+        if (pp > buf->f.max_plane)
         {
-            size += hb_image_stride( buf->f.fmt, width, p ) * 
-                    hb_image_height_stride( buf->f.fmt, height, p );
+            buf->f.max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride(buf->f.fmt, width, pp) *
+                    hb_image_height_stride(buf->f.fmt, height, pp );
         }
     }
 
@@ -630,7 +942,7 @@ void hb_video_buffer_realloc( hb_buffer_t * buf, int width, int height )
     buf->f.height = height;
     buf->size = size;
 
-    hb_buffer_init_planes_internal( buf, has_plane );
+    hb_buffer_init_planes(buf);
 }
 
 // this routine 'moves' data from src to dst by interchanging 'data',
@@ -656,9 +968,19 @@ void hb_buffer_close( hb_buffer_t ** _b )
 
     while( b )
     {
-#ifdef USE_QSV
+#if HB_PROJECT_FEATURE_QSV
         // Reclaim QSV resources before dropping the buffer.
         // when decoding without QSV, the QSV atom will be NULL.
+        if (b->storage != NULL && b->qsv_details.ctx != NULL)
+        {
+            AVFrame *frame = (AVFrame *)b->storage;
+            mfxFrameSurface1 *surface = (mfxFrameSurface1 *)frame->data[3];
+            if (surface)
+            {
+                hb_qsv_release_surface_from_pool_by_surface_pointer(b->qsv_details.qsv_frames_ctx, surface);
+                frame->data[3] = 0;
+            }
+        }
         if (b->qsv_details.qsv_atom != NULL && b->qsv_details.ctx != NULL)
         {
             hb_qsv_stage *stage = hb_qsv_get_last_stage(b->qsv_details.qsv_atom);
@@ -675,7 +997,7 @@ void hb_buffer_close( hb_buffer_t ** _b )
                 }
             }
             hb_qsv_flush_stages(b->qsv_details.ctx->pipes,
-                                (hb_qsv_list**)&b->qsv_details.qsv_atom);
+                                (hb_qsv_list**)&b->qsv_details.qsv_atom, 1);
         }
 #endif
 
@@ -689,7 +1011,24 @@ void hb_buffer_close( hb_buffer_t ** _b )
         hb_list_rem(buffers.alloc_list, b);
         hb_unlock(buffers.lock);
 #endif
-        if( buffer_pool && b->data && !hb_fifo_is_full( buffer_pool ) )
+        if (b->storage_type == AVFRAME)
+        {
+            av_frame_unref((AVFrame *)b->storage);
+            av_frame_free((AVFrame **)&b->storage);
+        }
+#ifdef __APPLE__
+        else if (b->storage_type == COREMEDIA)
+        {
+            CFRelease((CMSampleBufferRef)b->storage);
+        }
+#endif
+        if (b->storage_type != AVFRAME)
+        {
+            hb_buffer_wipe_side_data(b);
+            av_freep(&b->side_data);
+        }
+
+        if (buffer_pool && !hb_fifo_is_full(buffer_pool))
         {
 #if defined(HB_BUFFER_DEBUG)
             if (hb_fifo_contains(buffer_pool, b))
@@ -703,10 +1042,10 @@ void hb_buffer_close( hb_buffer_t ** _b )
             continue;
         }
         // either the pool is full or this size doesn't use a pool
-        // free the buf 
-        if( b->data )
+        // free the buf
+        if (b->data && b->storage_type == STANDARD)
         {
-            free(b->data);
+            av_free(b->data);
             hb_lock(buffers.lock);
             buffers.allocated -= b->alloc;
             hb_unlock(buffers.lock);
@@ -720,42 +1059,40 @@ void hb_buffer_close( hb_buffer_t ** _b )
 
 hb_image_t * hb_image_init(int pix_fmt, int width, int height)
 {
-    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
-    int p;
-    uint8_t has_plane[4] = {0,};
+    const AVPixFmtDescriptor * desc = av_pix_fmt_desc_get(pix_fmt);
+    uint8_t                    has_plane[4] = {0,};
+    int                        ii, pp;
 
     if (desc == NULL)
     {
         return NULL;
     }
-    for (p = 0; p < 4; p++)
-    {
-        has_plane[desc->comp[p].plane] = 1;
-    }
-
-    int size = 0;
-    for (p = 0; p < 4; p++)
-    {
-        if (has_plane[p])
-        {
-            size += hb_image_stride( pix_fmt, width, p ) * 
-                    hb_image_height_stride( pix_fmt, height, p );
-        }
-    }
 
     hb_image_t *image = calloc(1, sizeof(hb_image_t));
     if (image == NULL)
     {
         return NULL;
     }
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-    image->data  = malloc(size);
-#elif defined( SYS_CYGWIN )
-    /* FIXME */
-    image->data  = malloc(size + 17);
-#else
-    image->data  = memalign(16, size);
-#endif
+
+    int size = 0;
+    for (ii = 0; ii < desc->nb_components; ii++)
+    {
+        // For non-planar formats, comp[ii].plane can contain the
+        // same value for multiple comp.
+        pp = desc->comp[ii].plane;
+        if (pp > image->max_plane)
+        {
+            image->max_plane = pp;
+        }
+        if (!has_plane[pp])
+        {
+            has_plane[pp] = 1;
+            size += hb_image_stride( pix_fmt, width, pp ) *
+                    hb_image_height_stride( pix_fmt, height, pp );
+        }
+    }
+
+    image->data  = av_malloc(size);
     if (image->data == NULL)
     {
         free(image);
@@ -766,21 +1103,18 @@ hb_image_t * hb_image_init(int pix_fmt, int width, int height)
     image->height = height;
     memset(image->data, 0, size);
 
-    uint8_t * plane = image->data;
-    for (p = 0; p < 4; p++)
+    uint8_t * data = image->data;
+    for (pp = 0; pp <= image->max_plane; pp++)
     {
-        if (has_plane[p])
-        {
-            image->plane[p].data = plane;
-            image->plane[p].stride = hb_image_stride(pix_fmt, width, p );
-            image->plane[p].height_stride =
-                                    hb_image_height_stride(pix_fmt, height, p );
-            image->plane[p].width  = hb_image_width(pix_fmt, width, p );
-            image->plane[p].height = hb_image_height(pix_fmt, height, p );
-            image->plane[p].size   =
-                        image->plane[p].stride * image->plane[p].height_stride;
-            plane += image->plane[p].size;
-        }
+        image->plane[pp].data   = data;
+        image->plane[pp].stride = hb_image_stride(pix_fmt, width, pp);
+        image->plane[pp].height_stride =
+                                hb_image_height_stride(pix_fmt, height, pp);
+        image->plane[pp].width  = hb_image_width(pix_fmt, width, pp);
+        image->plane[pp].height = hb_image_height(pix_fmt, height, pp);
+        image->plane[pp].size   = image->plane[pp].stride *
+                                  image->plane[pp].height_stride;
+        data                   += image->plane[pp].size;
     }
     return image;
 }
@@ -789,14 +1123,7 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
 {
     hb_image_t *image = calloc(1, sizeof(hb_image_t));
 
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
-    image->data  = malloc( buf->size );
-#elif defined( SYS_CYGWIN )
-    /* FIXME */
-    image->data  = malloc( buf->size + 17 );
-#else
-    image->data  = memalign( 16, buf->size );
-#endif
+    image->data  = av_malloc( buf->size );
     if (image->data == NULL)
     {
         free(image);
@@ -806,11 +1133,13 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
     image->format = buf->f.fmt;
     image->width = buf->f.width;
     image->height = buf->f.height;
-    memcpy(image->data, buf->data, buf->size);
+    image->color_prim     = buf->f.color_prim;
+    image->color_transfer = buf->f.color_transfer;
+    image->color_matrix   = buf->f.color_matrix;
 
     int p;
     uint8_t *data = image->data;
-    for (p = 0; p < 4; p++)
+    for (p = 0; p <= buf->f.max_plane; p++)
     {
         image->plane[p].data = data;
         image->plane[p].width = buf->plane[p].width;
@@ -818,6 +1147,9 @@ hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
         image->plane[p].stride = buf->plane[p].stride;
         image->plane[p].height_stride = buf->plane[p].height_stride;
         image->plane[p].size = buf->plane[p].size;
+
+        memcpy(image->plane[p].data, buf->plane[p].data, buf->plane[p].size);
+
         data += image->plane[p].size;
     }
     return image;
@@ -831,7 +1163,7 @@ void hb_image_close(hb_image_t **_image)
     hb_image_t * image = *_image;
     if (image != NULL)
     {
-        free(image->data);
+        av_free(image->data);
         free(image);
         *_image = NULL;
     }
@@ -1146,7 +1478,7 @@ void hb_fifo_push_head( hb_fifo_t * f, hb_buffer_t * b )
     if( f->size > 0 )
     {
         tmp->next = f->first;
-    } 
+    }
     else
     {
         f->last = tmp;
