@@ -1,13 +1,14 @@
 /* vfr.c
 
-   Copyright (c) 2003-2018 HandBrake Team
+   Copyright (c) 2003-2023 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
-#include "hb.h"
+#include "libavutil/avutil.h"
+#include "handbrake/handbrake.h"
 
 //#define HB_DEBUG_CFR_DROPS 1
 #define MAX_FRAME_ANALYSIS_DEPTH 10
@@ -34,6 +35,7 @@ struct hb_filter_private_s
 
     // Duplicate frame detection members
     int             frame_analysis_depth;
+    int64_t         frame_analysis_duration;
     hb_list_t     * frame_rate_list;
     double        * frame_metric;
 
@@ -138,19 +140,55 @@ static void delete_metric(double * metrics, int pos, int size)
     memmove(dst, src, msize);
 }
 
-static int find_drop_frame(double *metrics, int count)
+static int find_drop_frame(hb_filter_private_t * pv, int count)
 {
-    int ii, min;
+    int           ii, min;
+    double      * metrics = pv->frame_metric;
+    hb_buffer_t * buf, * first;
+    double        cfr_stop;
 
+    // compute where the second to last frame in the frame_rate_list would
+    // stop if the frame rate were constant.
+    //
+    // this is our target stopping time for CFR and earliest possible
+    // stopping time for PFR.
+    cfr_stop = pv->out_last_stop + pv->frame_duration * (count - 1);
+
+    // If the last frame's stop timestamp is before the calculated
+    // CFR stop time of the second to last frame, then we need to drop a frame.
+    buf   = hb_list_item(pv->frame_rate_list, count - 1);
+
+    // Shortcut exit when entire list is CFR OK
+    if (buf->s.stop >= (int64_t)cfr_stop)
+    {
+        return -1;
+    }
+
+    first = hb_list_item(pv->frame_rate_list, 0);
     min = 0;
     for (ii = 1; ii < count; ii++)
     {
-        if (metrics[ii] < metrics[min])
+        hb_buffer_t * buf = hb_list_item(pv->frame_rate_list, ii);
+        // Don't check buffers outside analysis window
+        if (buf->s.stop - first->s.start > pv->frame_analysis_duration)
+        {
+            break;
+        }
+        if (min < 0 || metrics[ii] < metrics[min])
         {
             min = ii;
         }
     }
 
+    cfr_stop = pv->out_last_stop + pv->frame_duration * (ii - 1);
+    buf      = hb_list_item(pv->frame_rate_list, ii - 1);
+
+    // Don't drop buffers if frames fit in CRF time bounds
+    if (buf->s.stop >= (int64_t)cfr_stop)
+    {
+        return -1;
+    }
+
     return min;
 }
 
@@ -177,8 +215,11 @@ static hb_buffer_t * adjust_frame_rate( hb_filter_private_t * pv,
 
     if (pv->cfr == 0)
     {
-        ++pv->count_frames;
-        pv->out_last_stop = in->s.stop;
+        if (in)
+        {
+            ++pv->count_frames;
+            pv->out_last_stop = in->s.stop;
+        }
         return in;
     }
 
@@ -220,26 +261,23 @@ static hb_buffer_t * adjust_frame_rate( hb_filter_private_t * pv,
     }
 
     hb_buffer_list_t   list;
-    hb_buffer_t      * out, * last;
+    hb_buffer_t      * out;
     double             cfr_stop;
+    int                drop_frame;
 
     hb_buffer_list_clear(&list);
 
-    // compute where the second to last frame in the frame_rate_list would
-    // stop if the frame rate were constant.
-    //
-    // this is our target stopping time for CFR and earliest possible
-    // stopping time for PFR.
-    cfr_stop = pv->out_last_stop + pv->frame_duration * (count - 1);
-
-    // If the last frame's stop timestamp is before the calculated
-    // CFR stop time of the second to last frame, then we need to drop a frame.
-    last = hb_list_item(pv->frame_rate_list, count - 1);
-    if (last->s.stop < cfr_stop)
+    drop_frame = find_drop_frame(pv, count);
+    if (drop_frame >= 0)
     {
-        int drop_frame;
 
-        drop_frame = find_drop_frame(pv->frame_metric, count);
+        // We may have to drop multiple frames.  Pick frames to drop
+        // that appear to have minimum motion.
+        //
+        // The main use case we are trying to cover here is
+        // "progressive telecine" where there is a repeating pattern
+        // of a new frame followed by some number of repeated frames.
+        // We want to keep the "new frames" and drop the repeats.
         out = hb_list_item(pv->frame_rate_list, drop_frame);
 
 #if defined(HB_DEBUG_CFR_DROPS)
@@ -361,15 +399,14 @@ static int hb_vfr_init(hb_filter_object_t *filter, hb_filter_init_t *init)
         // in_vrate / out_vrate tells us how many consecutive repeated
         // frames we can expect to see.  if the number of consecutive
         // repeated frame is < 2, we need the number of consecutive
-        // non-repeated frames.  Then add 1 to round up fractions
-        // and add 2 so that we should have transitions that we can
-        // detect at both ends of a sequence.
+        // non-repeated frames.  Then add 1 so that we should have
+        // transitions that we can detect at both ends of a sequence.
         double factor = in_vrate / out_vrate;
-        if (factor >= 1.0 && factor < 2.0)
+        if (factor > 1.0 && factor < 2.0)
         {
             factor = 1 / (factor - 1);
         }
-        pv->frame_analysis_depth = factor + 3;
+        pv->frame_analysis_depth = ceil(factor) + 1;
 
         // if we end up with an absurdly large value, limit it
         if (pv->frame_analysis_depth > MAX_FRAME_ANALYSIS_DEPTH)
@@ -377,6 +414,7 @@ static int hb_vfr_init(hb_filter_object_t *filter, hb_filter_init_t *init)
             pv->frame_analysis_depth = MAX_FRAME_ANALYSIS_DEPTH;
         }
     }
+    pv->frame_analysis_duration = pv->frame_analysis_depth * 90000 / in_vrate;
     pv->frame_metric = calloc(pv->frame_analysis_depth, sizeof(double));
     pv->frame_metric[0] = INT_MAX;
 
@@ -439,7 +477,7 @@ static hb_filter_info_t * hb_vfr_info( hb_filter_object_t * filter )
     info->human_readable_desc = malloc(128);
     info->human_readable_desc[0] = 0;
 
-    info->out.vrate      = pv->input_vrate;
+    info->output.vrate      = pv->input_vrate;
     if (pv->cfr == 2)
     {
         // For PFR, we want the framerate based on the source's actual
@@ -450,14 +488,14 @@ static hb_filter_info_t * hb_vfr_info( hb_filter_object_t * filter )
         {
             // peak framerate is lower than the source framerate.
             // so signal that the framerate will be the peak fps.
-            info->out.vrate = pv->vrate;
+            info->output.vrate = pv->vrate;
         }
     }
     else
     {
-        info->out.vrate = pv->vrate;
+        info->output.vrate = pv->vrate;
     }
-    info->out.cfr = pv->cfr;
+    info->output.cfr = pv->cfr;
     if ( pv->cfr == 0 )
     {
         /* Ensure we're using "Same as source" FPS */
@@ -500,6 +538,11 @@ static void hb_vfr_close( hb_filter_object_t * filter )
         hb_log("vfr: %d frames output, %d dropped and %d duped for CFR/PFR",
                pv->count_frames, pv->drops, pv->dups );
     }
+    else
+    {
+        hb_log("vfr: %d frames output, %d dropped",
+               pv->count_frames, pv->drops);
+    }
 
     if( pv->job )
     {
